@@ -1,14 +1,12 @@
 """Geospatial transforms"""
 
-from __future__ import division
-
+from collections.abc import Iterable
 import math
+import sys
 
 from affine import Affine
 
 from rasterio._transform import _transform_from_gcps
-from rasterio.compat import Iterable
-
 
 IDENTITY = Affine.identity()
 GDAL_IDENTITY = IDENTITY.to_gdal()
@@ -152,15 +150,10 @@ def xy(transform, rows, cols, offset='center'):
     ys : list
         y coordinates in coordinate reference system
     """
-
-    single_col = False
-    single_row = False
     if not isinstance(cols, Iterable):
         cols = [cols]
-        single_col = True
     if not isinstance(rows, Iterable):
         rows = [rows]
-        single_row = True
 
     if offset == 'center':
         coff, roff = (0.5, 0.5)
@@ -177,16 +170,15 @@ def xy(transform, rows, cols, offset='center'):
 
     xs = []
     ys = []
-    for col, row in zip(cols, rows):
-        x, y = transform * transform.translation(coff, roff) * (col, row)
+    T = transform * transform.translation(coff, roff)
+    for pt in zip(cols, rows):
+        x, y = T * pt
         xs.append(x)
         ys.append(y)
 
-    if single_row:
-        ys = ys[0]
-    if single_col:
-        xs = xs[0]
-
+    if len(xs) == 1:
+        # xs and ys will always have the same length
+        return xs[0], ys[0]
     return xs, ys
 
 
@@ -210,8 +202,9 @@ def rowcol(transform, xs, ys, op=math.floor, precision=None):
     op : function
         Function to convert fractional pixels to whole numbers (floor, ceiling,
         round)
-    precision : int, optional
-        Decimal places of precision in indexing, as in `round()`.
+    precision : int or float, optional
+        An integer number of decimal points of precision when computing
+        inverse transform, or an absolute float precision.
 
     Returns
     -------
@@ -221,34 +214,34 @@ def rowcol(transform, xs, ys, op=math.floor, precision=None):
         list of column indices
     """
 
-    single_x = False
-    single_y = False
     if not isinstance(xs, Iterable):
         xs = [xs]
-        single_x = True
     if not isinstance(ys, Iterable):
         ys = [ys]
-        single_y = True
 
     if precision is None:
-        eps = 0.0
+        eps = sys.float_info.epsilon
+    elif isinstance(precision, int):
+        eps = 10.0 ** -precision
     else:
-        eps = 10.0 ** -precision * (1.0 - 2.0 * op(0.1))
+        eps = precision
+
+    # If op rounds up, switch the sign of eps.
+    if op(0.1) >= 1:
+        eps = -eps
 
     invtransform = ~transform
 
     rows = []
     cols = []
     for x, y in zip(xs, ys):
-        fcol, frow = invtransform * (x + eps, y - eps)
+        fcol, frow = invtransform * (x + eps, y + eps)
         cols.append(op(fcol))
         rows.append(op(frow))
 
-    if single_x:
-        cols = cols[0]
-    if single_y:
-        rows = rows[0]
-
+    if len(cols) == 1:
+        # rows and cols will always have the same length
+        return rows[0], cols[0]
     return rows, cols
 
 
