@@ -17,24 +17,19 @@ The newer float precision read-write window capabilities of Rasterio
 require instances of Window to be used.
 """
 
-from __future__ import division
 import collections
+from collections.abc import Iterable
 import functools
 import math
-import warnings
 
-import attr
 from affine import Affine
+import attr
 import numpy as np
 
-from rasterio.compat import Iterable
 from rasterio.errors import WindowError
 from rasterio.transform import rowcol, guard_transform
 
 
-PIXEL_PRECISION = 6
-
-
 class WindowMethodsMixin(object):
     """Mixin providing methods for window-related calculations.
     These methods are wrappers for the functionality in
@@ -69,7 +64,6 @@ class WindowMethodsMixin(object):
         window: Window
         """
         transform = guard_transform(self.transform)
-
         return from_bounds(
             left, bottom, right, top, transform=transform,
             height=self.height, width=self.width, precision=precision)
@@ -252,22 +246,30 @@ def intersect(*windows):
     return True
 
 
-def from_bounds(left, bottom, right, top, transform=None,
-                height=None, width=None, precision=None):
+def from_bounds(
+    left, bottom, right, top, transform=None, height=None, width=None, precision=None
+):
     """Get the window corresponding to the bounding coordinates.
 
     Parameters
     ----------
-    left, bottom, right, top: float
-        Left (west), bottom (south), right (east), and top (north)
-        bounding coordinates.
+    left: float, required
+        Left (west) bounding coordinates
+    bottom: float, required
+        Bottom (south) bounding coordinates
+    right: float, required
+        Right (east) bounding coordinates
+    top: float, required
+        Top (north) bounding coordinates
     transform: Affine, required
         Affine transform matrix.
-    height, width: int, required
-        Number of rows and columns of the window.
-    precision: int, optional
-        Number of decimal points of precision when computing inverse
-        transform.
+    height: int, required
+        Number of rows of the window.
+    width: int, required
+        Number of columns of the window.
+    precision: int or float, optional
+        An integer number of decimal points of precision when computing
+        inverse transform, or an absolute float precision.
 
     Returns
     -------
@@ -283,15 +285,22 @@ def from_bounds(left, bottom, right, top, transform=None,
     if not isinstance(transform, Affine):  # TODO: RPCs?
         raise WindowError("A transform object is required to calculate the window")
 
-    row_start, col_start = rowcol(
-        transform, left, top, op=float, precision=precision)
+    rows, cols = rowcol(
+        transform,
+        [left, right, right, left],
+        [top, top, bottom, bottom],
+        op=float,
+        precision=precision,
+    )
+    row_start, row_stop = min(rows), max(rows)
+    col_start, col_stop = min(cols), max(cols)
 
-    row_stop, col_stop = rowcol(
-        transform, right, bottom, op=float, precision=precision)
-
-    return Window.from_slices(
-        (row_start, row_stop), (col_start, col_stop), height=height,
-        width=width, boundless=True)
+    return Window(
+        col_off=col_start,
+        row_off=row_start,
+        width=max(col_stop - col_start, 0.0),
+        height=max(row_stop - row_start, 0.0),
+    )
 
 
 def transform(window, transform):
@@ -663,17 +672,19 @@ class Window(object):
         -------
         Window
         """
-        operator = getattr(math, op, None)
-        if not operator:
-            raise WindowError("operator must be 'ceil' or 'floor'")
-        else:
-            return Window(
-                self.col_off, self.row_off,
-                operator(round(self.width, pixel_precision) if
-                         pixel_precision is not None else self.width),
-                operator(round(self.height, pixel_precision) if
-                         pixel_precision is not None else self.height))
+        if op not in {'ceil', 'floor'}:
+            raise WindowError("operator must be 'ceil' or 'floor', got '{}'".format(op))
 
+        operator = getattr(math, op)
+        if pixel_precision is None:
+            return Window(self.col_off, self.row_off,
+                          operator(self.width), operator(self.height))
+        else:
+            return Window(self.col_off, self.row_off,
+                          operator(round(self.width, pixel_precision)),
+                          operator(round(self.height, pixel_precision)))
+
+    # TODO: deprecate round_shape at 1.3.0, with a warning.
     round_shape = round_lengths
 
     def round_offsets(self, op='floor', pixel_precision=None):
@@ -693,16 +704,17 @@ class Window(object):
         -------
         Window
         """
-        operator = getattr(math, op, None)
-        if not operator:
-            raise WindowError("operator must be 'ceil' or 'floor'")
+        if op not in {'ceil', 'floor'}:
+            raise WindowError("operator must be 'ceil' or 'floor', got '{}'".format(op))
+
+        operator = getattr(math, op)
+        if pixel_precision is None:
+            return Window(operator(self.col_off), operator(self.row_off),
+                          self.width, self.height)
         else:
-            return Window(
-                operator(round(self.col_off, pixel_precision) if
-                         pixel_precision is not None else self.col_off),
-                operator(round(self.row_off, pixel_precision) if
-                         pixel_precision is not None else self.row_off),
-                self.width, self.height)
+            return Window(operator(round(self.col_off, pixel_precision)),
+                          operator(round(self.row_off, pixel_precision)),
+                          self.width, self.height)
 
     def crop(self, height, width):
         """Return a copy cropped to height and width"""
